use js_sys::Array;
use wasm_bindgen::{JsCast, JsValue};
use web_sys::{Blob, HtmlAnchorElement, Url};

use yew::prelude::*;
use yewdux::prelude::*;

use crate::components::states::{CurrentTransfer, MainState};

use crate::data::csv::state_to_csv;

type NoParamsCallback = Box<dyn Fn(()) -> ()>;

pub fn toggle_in_transfer_hashes_callback(
    main_dispatch: Dispatch<MainState>,
) -> Callback<web_sys::MouseEvent> {
    let main_dispatch = main_dispatch.clone();
    Callback::from(move |_| {
        main_dispatch.reduce_mut(|state| {
            state.preferences.in_transfer_hashes ^= true;
        })
    })
}

pub fn new_plate_dialog_callback(
    new_plate_dialog_is_open: UseStateHandle<bool>,
) -> NoParamsCallback {
    let new_plate_dialog_is_open = new_plate_dialog_is_open.clone();
    Box::new(move |_| {
        new_plate_dialog_is_open.set(false);
    })
}

pub fn open_new_plate_dialog_callback(
    new_plate_dialog_is_open: UseStateHandle<bool>,
) -> NoParamsCallback {
    let new_plate_dialog_is_open = new_plate_dialog_is_open.clone();
    Box::new(move |_| {
        new_plate_dialog_is_open.set(true);
    })
}

pub fn new_button_callback(
    main_dispatch: Dispatch<MainState>,
    ct_dispatch: Dispatch<CurrentTransfer>,
) -> Callback<web_sys::MouseEvent> {
    Callback::from(move |_| {
        let window = web_sys::window().unwrap();
        let confirm =
            window.confirm_with_message("This will reset all plates and transfers. Proceed?");
        if let Ok(confirm) = confirm {
            if confirm {
                main_dispatch.set(MainState::default());
                ct_dispatch.set(CurrentTransfer::default());
            }
        }
    })
}

fn save_str(data: &str, name: &str) {
    let blob =
        Blob::new_with_str_sequence(&Array::from_iter(std::iter::once(JsValue::from_str(data))));
    if let Ok(blob) = blob {
        let url = Url::create_object_url_with_blob(&blob).expect("We have a blob, why not URL?");
        // Beneath is the cool hack to download files
        let window = web_sys::window().unwrap();
        let document = window.document().unwrap();
        let anchor = document
            .create_element("a")
            .unwrap()
            .dyn_into::<HtmlAnchorElement>()
            .unwrap();
        anchor.set_download(name);
        anchor.set_href(&url);
        anchor.click();
    }
}

pub fn export_csv_button_callback(main_state: std::rc::Rc<MainState>) -> Callback<MouseEvent> {
    Callback::from(move |_| {
        if main_state.transfers.is_empty() {
            web_sys::window()
                .unwrap()
                .alert_with_message("No transfers to export.")
                .unwrap();
            return;
        }
        web_sys::window().unwrap().alert_with_message("CSV export is currently not importable. Export as JSON if you'd like to back up your work!").unwrap();
        if let Ok(csv) = state_to_csv(&main_state) {
            save_str(&csv, "transfers.csv");
        }
    })
}

pub fn export_json_button_callback(main_state: std::rc::Rc<MainState>) -> Callback<MouseEvent> {
    Callback::from(move |_| {
        if let Ok(json) = serde_json::to_string(&main_state) {
            save_str(&json, "plate-tool-state.json");
        } else {
            web_sys::window()
                .unwrap()
                .alert_with_message("Failed to export.")
                .unwrap();
        }
    })
}
